#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for eos_prefix_lists
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
---
module: eos_prefix_lists
short_description: Manages Prefix lists resource module
description: This module configures and manages the attributes of Prefix lists on Arista
  EOS platforms.
version_added: 2.2.0
author: Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
- Tested against Arista EOS 4.24.6F
- This module works with connection C(network_cli). See the L(EOS Platform Options,eos_platform_options).
options:
   config:
      description: A list of dictionary of prefix-list options
      type: list
      elements: dict
      suboptions:
        afi:
          description:
          - The Address Family Indicator (AFI) for the  prefix list.
          type: str
          required: true
          choices:
          - ipv4
          - ipv6
        prefix_lists:
          description:
          - A list of prefix-lists.
          type: list
          elements: dict
          suboptions:
            name:
              description: Name of the prefix-list
              type: str
              required: true
            entries:
              description: List of prefix-lists
              type: list
              elements: dict
              suboptions:
                action:
                  description: action to be performed on the specified path
                  type: str
                  choices: ['deny', 'permit']
                address:
                  description: ipv4/v6 address in prefix-mask or address-masklen format
                  type: str
                match:
                  description: match masklen
                  type: dict
                  suboptions:
                    operator:
                      description: equalto/greater than/lesser than
                      type: str
                      choices: ['eq', 'le', 'ge']
                    masklen:
                      description: Mask Length.
                      type: int
                sequence:
                  description: sequence number
                  type: int
                resequence:
                  description: Resequence the list.
                  type: dict
                  suboptions:
                    default:
                      description: Resequence with default values (10).
                      type: bool
                    start_seq:
                      description: Starting sequence number.
                      type: int
                    step:
                      description: Step to increment the sequence number.
                      type: int
   running_config:
      description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the EOS device by
        executing the command B(show running-config | section access-list).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
      type: str
   state:
      description:
      - The state the configuration should be left in.
      type: str
      choices:
      - deleted
      - merged
      - overridden
      - replaced
      - gathered
      - rendered
      - parsed
      default: merged
"""
EXAMPLES = """
# Using merged
# Before state
# veos#show running-config | section prefix-lists
# veos#

  - name: Merge provided configuration with device configuration
    arista.eos.eos_prefix_lists:
      config:
        - afi: "ipv4"
          prefix_lists:
            - name: "v401"
              entries:
                - sequence: 25
                  action: "deny"
                  address: "45.55.4.0/24"
                - sequence: 100
                  action: "permit"
                  address: "11.11.2.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
            - name: "v402"
              entries:
                - action: "deny"
                  address: "10.1.1.0/24"
                  sequence: 10
                  match:
                    masklen: 32
                    operator: "ge"
        - afi: "ipv6"
          prefix_lists:
            - name: "v601"
              entries:
                - sequence: 125
                  action: "deny"
                  address: "5000:1::/64"

# After State
# veos#
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#
#
# Module Execution:
# "after": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ],
#     "before": {},
#     "changed": true,
#     "commands": [
#         "ipv6 prefix-list v601",
#         "seq 125 deny 5000:1::/64",
#         "ip prefix-list v401",
#         "seq 25 deny 45.55.4.0/24",
#         "seq 100 permit 11.11.2.0/24 ge 32",
#         "ip prefix-list v402",
#         "seq 10 deny 10.1.1.0/24 ge 32"
#     ],
#

# using merged:
# Failure scenario : 'merged' should not be used when an existing prefix-list (sequence number)
# is to be modified.

# Before State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

  - name: Merge provided configuration with device configuration
    arista.eos.eos_prefix_lists:
      config:
        - afi: "ipv4"
          prefix_lists:
            - name: "v401"
              entries:
                - sequence: 25
                  action: "deny"
                  address: "45.55.4.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
                - sequence: 100
                  action: "permit"
                  address: "11.11.2.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
            - name: "v402"
              entries:
                - action: "deny"
                  address: "10.1.1.0/24"
                  sequence: 10
                  match:
                    masklen: 32
                    operator: "ge"
        - afi: "ipv6"
          prefix_lists:
            - name: "v601"
              entries:
                - sequence: 125
                  action: "deny"
                  address: "5000:1::/64"
      state: merged

# Module Execution:
# fatal: [192.168.122.113]: FAILED! => {
#     "changed": false,
#     "invocation": {
#         "module_args": {
#             "config": [
#                 {
#                     "afi": "ipv4",
#                     "prefix_lists": [
#                         {
#                             "entries": [
#                                 {
#                                     "action": "deny",
#                                     "address": "45.55.4.0/24",
#                                     "match": {
#                                         "masklen": 32,
#                                         "operator": "ge"
#                                     },
#                                     "resequence": null,
#                                     "sequence": 25
#                                 },
#                                 {
#                                     "action": "permit",
#                                     "address": "11.11.2.0/24",
#                                     "match": {
#                                         "masklen": 32,
#                                         "operator": "ge"
#                                     },
#                                     "resequence": null,
#                                     "sequence": 100
#                                 }
#                             ],
#                             "name": "v401"
#                         },
#                         {
#                             "entries": [
#                                 {
#                                     "action": "deny",
#                                     "address": "10.1.1.0/24",
#                                     "match": {
#                                         "masklen": 32,
#                                         "operator": "ge"
#                                     },
#                                     "resequence": null,
#                                     "sequence": 10
#                                 }
#                             ],
#                             "name": "v402"
#                         }
#                     ]
#                 },
#                 {
#                     "afi": "ipv6",
#                     "prefix_lists": [
#                         {
#                             "entries": [
#                                 {
#                                     "action": "deny",
#                                     "address": "5000:1::/64",
#                                     "match": null,
#                                     "resequence": null,
#                                     "sequence": 125
#                                 }
#                             ],
#                             "name": "v601"
#                         }
#                     ]
#                 }
#             ],
#             "running_config": null,
#             "state": "merged"
#         }
#     },
#     "msg": "Sequence number 25 is already present. Use replaced/overridden operation to change the configuration"
# }
#

# Using Replaced:

# Before state:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#
  - name: Replace
    arista.eos.eos_prefix_lists:
      config:
        - afi: "ipv4"
          prefix_lists:
            - name: "v401"
              entries:
                - sequence: 25
                  action: "deny"
                  address: "45.55.4.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
                - sequence: 200
                  action: "permit"
                  address: "200.11.2.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
      state: replaced
# After State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 200 permit 200.11.2.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#
#
#
# Module Execution:
#
# "after": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "200.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 200
#                         }
#                     ],
#                     "name": "v401"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ],
#     "before": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "ip prefix-list v401",
#         "no seq 25",
#         "seq 25 deny 45.55.4.0/24 ge 32",
#         "seq 200 permit 200.11.2.0/24 ge 32",
#         "no seq 100",
#         "no ip prefix-list v402"
#     ],

# Using overridden:
# Before State:

# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 200 permit 200.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#


  - name: Override
    arista.eos.eos_prefix_lists:
      config:
        - afi: "ipv4"
          prefix_lists:
            - name: "v401"
              entries:
                - sequence: 25
                  action: "deny"
                  address: "45.55.4.0/24"
                - sequence: 300
                  action: "permit"
                  address: "30.11.2.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
            - name: "v403"
              entries:
                - action: "deny"
                  address: "10.1.1.0/24"
                  sequence: 10
      state: overridden

# After State
# veos#
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
# veos#
#
#
# Module Execution:
# "after": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "30.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 300
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v403"
#                 }
#             ]
#         }
#     ],
#     "before": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         },
#                         {
#                             "action": "permit",
#                             "address": "200.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 200
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "no ipv6 prefix-list v601",
#         "ip prefix-list v401",
#         "seq 25 deny 45.55.4.0/24",
#         "seq 300 permit 30.11.2.0/24 ge 32",
#         "no seq 100",
#         "no seq 200",
#         "ip prefix-list v403",
#         "seq 10 deny 10.1.1.0/24",
#         "no ip prefix-list v402"
#     ],
#

# Using deleted:
# Before State:

# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

  - name: Delete device configuration
    arista.eos.eos_prefix_lists:
      config:
        - afi: "ipv6"
      state: deleted


# after State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
#
#
# Module Execution:
#     "after": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         },
#                         {
#                             "action": "permit",
#                             "address": "30.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 300
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v403"
#                 }
#             ]
#         }
#     ],
#     "before": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         },
#                         {
#                             "action": "permit",
#                             "address": "30.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 300
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v403"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ],
#     "changed": true,
#     "commands": [
#         "no ipv6 prefix-list v601"
#     ],
#

# Using deleted
# Before state:

# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
# veos#

  - name: Delete device configuration
    arista.eos.eos_prefix_lists:
      state: deleted

# After State:
# veos#show running-config | section prefix-list
# veos#
#
# Module Execution:
#    "after": {},
#    "before": [
#        {
#            "afi": "ipv4",
#            "prefix_lists": [
#                {
#                    "entries": [
#                        {
#                            "action": "deny",
#                            "address": "45.55.4.0/24",
#                            "match": {
#                                "masklen": 32,
#                                "operator": "ge"
#                            },
#                            "sequence": 25
#                        },
#                        {
#                            "action": "permit",
#                            "address": "11.11.2.0/24",
#                            "match": {
#                                "masklen": 32,
#                                "operator": "ge"
#                            },
#                            "sequence": 100
#                        },
#                        {
#                            "action": "permit",
#                            "address": "30.11.2.0/24",
#                            "match": {
#                                "masklen": 32,
#                                "operator": "ge"
#                            },
#                            "sequence": 300
#                        }
#                    ],
#                    "name": "v401"
#                },
#                {
#                    "entries": [
#                        {
#                            "action": "deny",
#                            "address": "10.1.1.0/24",
#                            "match": {
#                                "masklen": 32,
#                                "operator": "ge"
#                            },
#                            "sequence": 10
#                        }
#                    ],
#                    "name": "v402"
#                },
#                {
#                    "entries": [
#                        {
#                            "action": "deny",
#                            "address": "10.1.1.0/24",
#                            "sequence": 10
#                        }
#                    ],
#                    "name": "v403"
#                }
#            ]
#        }
#    ],
#    "changed": true,
#    "commands": [
#        "no ip prefix-list v401",
#        "no ip prefix-list v402",
#        "no ip prefix-list v403"
#    ],
#

# Using parsed:
# parse_prefix_lists.cfg
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
#
  - name: parse configs
    arista.eos.eos_prefix_lists:
      running_config: "{{ lookup('file', './parsed_prefix_lists.cfg') }}"
      state: parsed

# Module Execution:
#     "parsed": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ]

# Using rendered:
  - name: Render provided configuration
    arista.eos.eos_prefix_lists:
      config:
        - afi: "ipv4"
          prefix_lists:
            - name: "v401"
              entries:
                - sequence: 25
                  action: "deny"
                  address: "45.55.4.0/24"
                - sequence: 200
                  action: "permit"
                  address: "200.11.2.0/24"
                  match:
                    masklen: 32
                    operator: "ge"
            - name: "v403"
              entries:
                - action: "deny"
                  address: "10.1.1.0/24"
                  sequence: 10
      state: rendered

# Module Execution:
#  "rendered": [
#         "ip prefix-list v401",
#         "seq 25 deny 45.55.4.0/24",
#         "seq 200 permit 200.11.2.0/24 ge 32",
#         "ip prefix-list v403",
#         "seq 10 deny 10.1.1.0/24"
#     ]
#

# using gathered:
# Device config:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

  - name: gather configs
    arista.eos.eos_prefix_lists:
      state: gathered

# Module Execution:
#
# "gathered": [
#         {
#             "afi": "ipv4",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "45.55.4.0/24",
#                             "sequence": 25
#                         },
#                         {
#                             "action": "permit",
#                             "address": "11.11.2.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 100
#                         }
#                     ],
#                     "name": "v401"
#                 },
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "10.1.1.0/24",
#                             "match": {
#                                 "masklen": 32,
#                                 "operator": "ge"
#                             },
#                             "sequence": 10
#                         }
#                     ],
#                     "name": "v402"
#                 }
#             ]
#         },
#         {
#             "afi": "ipv6",
#             "prefix_lists": [
#                 {
#                     "entries": [
#                         {
#                             "action": "deny",
#                             "address": "5000:1::/64",
#                             "sequence": 125
#                         }
#                     ],
#                     "name": "v601"
#                 }
#             ]
#         }
#     ],

"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.prefix_lists.prefix_lists import (
    Prefix_listsArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.prefix_lists.prefix_lists import (
    Prefix_lists,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Prefix_listsArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Prefix_lists(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The cisco.ios bgp_address_family fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    utils,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bgp_address_family import (
    Bgp_address_familyTemplate,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.bgp_address_family.bgp_address_family import (
    Bgp_address_familyArgs,
)


class Bgp_address_familyFacts(object):
    """The cisco.ios_bgp_address_family facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bgp_address_familyArgs.argument_spec

    def get_bgp_address_family_data(self, connection):
        return connection.get("show running-config | section ^router bgp")

    def _process_facts(self, objs):
        """makes data as per the facts after data obtained from parsers"""
        temp_af = []
        if objs.get("address_family"):
            for k, v in iteritems(objs["address_family"]):
                if (
                    k == "__"
                ):  # prepare dicts keys to operate on post spliting the keys
                    continue
                temp_dict = {}
                temp = [every for every in k.split("_") if every != ""]
                temp_dict["afi"] = temp.pop(0)
                if len(temp) > 1:
                    temp_dict["vrf"] = [
                        each.split(" ")[1] for each in temp if "vrf" in each
                    ][0]
                    temp_dict["safi"] = [
                        each for each in temp if "vrf" not in each
                    ][0]
                elif len(temp) == 1:
                    if "vrf" in temp[0]:
                        temp_dict["vrf"] = temp[0].split("vrf ")[1]
                    else:
                        temp_dict["safi"] = temp[0]
                neighbor = v.get("neighbor")

                if neighbor:

                    def _update_neighor_list(neighbor_list, temp, alter=None):
                        set = False
                        temp = utils.remove_empties(temp)
                        for each in neighbor_list:
                            for neighbor_type in [
                                "address",
                                "ipv6_adddress",
                                "tag",
                            ]:
                                try:
                                    if (
                                        neighbor_identifier
                                        == each[neighbor_type]
                                    ):
                                        each.update(temp)
                                        set = True
                                        break
                                except KeyError:
                                    continue
                        if not neighbor_list or not set:
                            if alter:
                                neighbor_list.extend(list(alter.values()))
                            else:
                                neighbor_list.append(temp)

                    neighbor_list, temp_param_list = [], []
                    temp, al = {}, {}
                    temp_param, neighbor_identifier = None, None

                    neighbor_type_list = ["address", "ipv6_adddress", "tag"]

                    for each in neighbor:
                        if temp_param and not each.get(temp_param) and temp:
                            temp.update({temp_param: temp_param_list})
                            _update_neighor_list(neighbor_list, temp)
                            temp_param_list = []
                            temp = {}
                        for neighbor_type in neighbor_type_list:
                            try:
                                temp[neighbor_type] = each[neighbor_type]
                                temp.update(each)
                                if not al.get(each.get(neighbor_type)):
                                    al[each.get(neighbor_type)] = each
                                else:
                                    al.get(each.get(neighbor_type)).update(
                                        each
                                    )
                                break
                            except KeyError:
                                continue
                        for param in [
                            "prefix_lists",
                            "route_maps",
                            "slow_peer",
                        ]:
                            param_val = each.get(param)
                            if param_val:
                                temp_param_list.append(param_val[0])
                                temp_param = param
                                break
                    if temp:
                        if temp_param:
                            temp.update({temp_param: temp_param_list})
                        _update_neighor_list(neighbor_list, temp, al)
                        temp_param_list = []
                        temp = {}
                    v["neighbor"] = neighbor_list
                v.update(temp_dict)
                temp_af.append(v)

            objs["address_family"] = temp_af

            objs["address_family"] = sorted(
                objs["address_family"], key=lambda k, sk="afi": k[sk]
            )
        return objs

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_address_family network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_bgp_address_family_data(connection)

        # parse native config using the Bgp_address_family template
        bgp_af_parser = Bgp_address_familyTemplate(
            lines=data.splitlines(), module=self._module
        )
        objs = bgp_af_parser.parse()

        if objs:

            objs = self._process_facts(utils.remove_empties(objs))

            ansible_facts["ansible_network_resources"].pop(
                "bgp_address_family", None
            )

            params = utils.remove_empties(
                bgp_af_parser.validate_config(
                    self.argument_spec, {"config": objs}, redact=True
                )
            )

            facts["bgp_address_family"] = params["config"]
            ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.ios.tests.unit.compat.mock import patch
from ansible_collections.cisco.ios.plugins.modules import ios_prefix_lists
from ansible_collections.cisco.ios.tests.unit.modules.utils import (
    set_module_args,
)
from .ios_module import TestIosModule, load_fixture


class TestIosPrefixListsModule(TestIosModule):
    module = ios_prefix_lists

    def setUp(self):
        super(TestIosPrefixListsModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_resource_connection_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base."
            "get_resource_connection"
        )
        self.get_resource_connection_config = (
            self.mock_get_resource_connection_config.start()
        )

        self.mock_get_resource_connection_facts = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base."
            "get_resource_connection"
        )
        self.get_resource_connection_facts = (
            self.mock_get_resource_connection_facts.start()
        )

        self.mock_edit_config = patch(
            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.providers.CliProvider.edit_config"
        )
        self.edit_config = self.mock_edit_config.start()

        self.mock_execute_show_command = patch(
            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.prefix_lists.prefix_lists."
            "Prefix_listsFacts.get_prefix_list_data"
        )
        self.execute_show_command = self.mock_execute_show_command.start()

    def tearDown(self):
        super(TestIosPrefixListsModule, self).tearDown()
        self.mock_get_resource_connection_config.stop()
        self.mock_get_resource_connection_facts.stop()
        self.mock_edit_config.stop()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_execute_show_command.stop()

    def load_fixtures(self, commands=None):
        def load_from_file(*args, **kwargs):
            return load_fixture("ios_prefix_lists.cfg")

        self.execute_show_command.side_effect = load_from_file

    def test_ios_prefix_lists_merged(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is merge test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="25.0.0.0/8",
                                        sequence=25,
                                    )
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is for prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="35.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                ],
                                name="test_prefix",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is merged ipv6 prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        le=100,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=20,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="merged",
            )
        )
        commands = [
            "ip prefix-list 10 description this is merge test",
            "ip prefix-list 10 seq 25 deny 25.0.0.0/8 ge 10 le 15",
            "ipv6 prefix-list test_ipv6 description this is merged ipv6 prefix-list",
            "ipv6 prefix-list test_ipv6 seq 20 deny 2001:DB8:0:4::/64 ge 80 le 100",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_prefix_lists_merged_idempotent(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is test description",
                                entries=[
                                    dict(
                                        action="deny",
                                        le=15,
                                        prefix="1.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=10,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=15,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        le=21,
                                        prefix="14.0.0.0/8",
                                        sequence=20,
                                    ),
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=50,
                                    )
                                ],
                                name="test",
                            ),
                            dict(
                                description="this is for prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="35.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                ],
                                name="test_prefix",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is ipv6 prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=10,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="merged",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_ios_prefix_lists_replaced(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is replace test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=15,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        le=21,
                                        prefix="14.0.0.0/8",
                                        sequence=20,
                                    ),
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is replace test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    )
                                ],
                                name="test_replace",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is ipv6 replace test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        le=100,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=10,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="replaced",
            )
        )
        commands = [
            "ip prefix-list 10 description this is replace test",
            "no ip prefix-list 10 seq 10 deny 35.0.0.0/8 ge 10",
            "no ip prefix-list 10 seq 5 deny 1.0.0.0/8 le 15",
            "ip prefix-list test_replace seq 10 deny 35.0.0.0/8 ge 20",
            "ip prefix-list test_replace description this is replace test",
            "no ipv6 prefix-list test_ipv6 seq 10 deny 2001:DB8:0:4::/64 ge 80",
            "ipv6 prefix-list test_ipv6 seq 10 deny 2001:DB8:0:4::/64 ge 80 le 100",
            "ipv6 prefix-list test_ipv6 description this is ipv6 replace test",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_prefix_lists_replaced_idempotent(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is test description",
                                entries=[
                                    dict(
                                        action="deny",
                                        le=15,
                                        prefix="1.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=10,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=15,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        le=21,
                                        prefix="14.0.0.0/8",
                                        sequence=20,
                                    ),
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=50,
                                    )
                                ],
                                name="test",
                            ),
                            dict(
                                description="this is for prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="35.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                ],
                                name="test_prefix",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is ipv6 prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=10,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="replaced",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_ios_prefix_lists_overridden(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is override test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=15,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        le=21,
                                        prefix="14.0.0.0/8",
                                        sequence=20,
                                    ),
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is override test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    )
                                ],
                                name="test_override",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is ipv6 override test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        le=100,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=10,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="overridden",
            )
        )
        commands = [
            "no ip prefix-list test",
            "no ip prefix-list test_prefix",
            "ip prefix-list 10 description this is override test",
            "no ip prefix-list 10 seq 10 deny 35.0.0.0/8 ge 10",
            "no ip prefix-list 10 seq 5 deny 1.0.0.0/8 le 15",
            "ip prefix-list test_override seq 10 deny 35.0.0.0/8 ge 20",
            "ip prefix-list test_override description this is override test",
            "no ipv6 prefix-list test_ipv6 seq 10 deny 2001:DB8:0:4::/64 ge 80",
            "ipv6 prefix-list test_ipv6 seq 10 deny 2001:DB8:0:4::/64 ge 80 le 100",
            "ipv6 prefix-list test_ipv6 description this is ipv6 override test",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_prefix_lists_overridden_idempotent(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is test description",
                                entries=[
                                    dict(
                                        action="deny",
                                        le=15,
                                        prefix="1.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=10,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=15,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        le=21,
                                        prefix="14.0.0.0/8",
                                        sequence=20,
                                    ),
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=15,
                                        prefix="12.0.0.0/8",
                                        sequence=50,
                                    )
                                ],
                                name="test",
                            ),
                            dict(
                                description="this is for prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="35.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                ],
                                name="test_prefix",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is ipv6 prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=10,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="overridden",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_ios_prefix_lists_delete_without_config(self):
        set_module_args(dict(state="deleted"))
        commands = [
            "no ip prefix-list test",
            "no ip prefix-list 10",
            "no ip prefix-list test_prefix",
            "no ipv6 prefix-list test_ipv6",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_prefix_lists_rendered(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        afi="ipv4",
                        prefix_lists=[
                            dict(
                                description="this is merge test",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="25.0.0.0/8",
                                        sequence=25,
                                    )
                                ],
                                name="10",
                            ),
                            dict(
                                description="this is for prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=10,
                                        le=15,
                                        prefix="35.0.0.0/8",
                                        sequence=5,
                                    ),
                                    dict(
                                        action="deny",
                                        ge=20,
                                        prefix="35.0.0.0/8",
                                        sequence=10,
                                    ),
                                ],
                                name="test_prefix",
                            ),
                        ],
                    ),
                    dict(
                        afi="ipv6",
                        prefix_lists=[
                            dict(
                                description="this is ipv6 prefix-list",
                                entries=[
                                    dict(
                                        action="deny",
                                        ge=80,
                                        le=100,
                                        prefix="2001:DB8:0:4::/64",
                                        sequence=10,
                                    )
                                ],
                                name="test_ipv6",
                            )
                        ],
                    ),
                ],
                state="rendered",
            )
        )
        commands = [
            "ip prefix-list 10 description this is merge test",
            "ip prefix-list 10 seq 25 deny 25.0.0.0/8 ge 10 le 15",
            "ip prefix-list test_prefix description this is for prefix-list",
            "ip prefix-list test_prefix seq 5 deny 35.0.0.0/8 ge 10 le 15",
            "ip prefix-list test_prefix seq 10 deny 35.0.0.0/8 ge 20",
            "ipv6 prefix-list test_ipv6 description this is ipv6 prefix-list",
            "ipv6 prefix-list test_ipv6 seq 10 deny 2001:DB8:0:4::/64 ge 80 le 100",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["rendered"]), sorted(commands))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: mnt_sessions_by_session_id_info
short_description: Information module for MNT Sessions By Session Id
description:
- Get all MNT Sessions By Session Id.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  session_id:
    description:
    - Session_id path parameter.
    type: str
requirements:
- ciscoisesdk >= 2.0.3
- python >= 3.5
notes:
  - SDK Method used are
    misc.Misc.get_sessions_by_session_id,

  - Paths used are
    get /Session/Active/SessionID/{session_id}/0,

"""

EXAMPLES = r"""
- name: Get all MNT Sessions By Session Id
  cisco.ise.mnt_sessions_by_session_id_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    session_id: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# Copyright (c), Felix Fontein <felix@fontein.de>, 2021
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import json
import os
import textwrap

import pytest

from mock import MagicMock

from ansible import constants as C
from ansible.errors import AnsibleError
from ansible.inventory.data import InventoryData
from ansible.inventory.manager import InventoryManager
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.internal_test_tools.tests.unit.mock.path import mock_unfrackpath_noop
from ansible_collections.community.internal_test_tools.tests.unit.mock.loader import DictDataLoader
from ansible_collections.community.internal_test_tools.tests.unit.utils.open_url_framework import (
    OpenUrlCall,
    OpenUrlProxy,
)

from ansible_collections.community.dns.plugins.inventory.hosttech_dns_records import InventoryModule


HOSTTECH_WSDL_DEFAULT_ENTRIES = [
    (125, 42, 'A', '', '1.2.3.4', 3600, None, None),
    (126, 42, 'A', '*', '1.2.3.5', 3600, None, None),
    (127, 42, 'AAAA', '', '2001:1:2::3', 3600, None, None),
    (128, 42, 'AAAA', 'foo', '2001:1:2::4', 3600, None, None),
    (129, 42, 'MX', '', 'example.com', 3600, None, '10'),
    (130, 42, 'CNAME', 'bar', 'example.org.', 10800, None, None),
]

HOSTTECH_JSON_DEFAULT_ENTRIES = [
    # (125, 42, 'A', '', '1.2.3.4', 3600, None, None),
    {
        'id': 125,
        'type': 'A',
        'name': '',
        'ipv4': '1.2.3.4',
        'ttl': 3600,
        'comment': '',
    },
    # (126, 42, 'A', '*', '1.2.3.5', 3600, None, None),
    {
        'id': 126,
        'type': 'A',
        'name': '*',
        'ipv4': '1.2.3.5',
        'ttl': 3600,
        'comment': '',
    },
    # (127, 42, 'AAAA', '', '2001:1:2::3', 3600, None, None),
    {
        'id': 127,
        'type': 'AAAA',
        'name': '',
        'ipv6': '2001:1:2::3',
        'ttl': 3600,
        'comment': '',
    },
    # (128, 42, 'AAAA', '*', '2001:1:2::4', 3600, None, None),
    {
        'id': 128,
        'type': 'AAAA',
        'name': 'foo',
        'ipv6': '2001:1:2::4',
        'ttl': 3600,
        'comment': '',
    },
    # (129, 42, 'MX', '', 'example.com', 3600, None, '10'),
    {
        'id': 129,
        'type': 'MX',
        'ownername': '',
        'name': 'example.com',
        'pref': 10,
        'ttl': 3600,
        'comment': '',
    },
    # (130, 42, 'CNAME', 'bar', 'example.org.', 10800, None, None),
    {
        'id': 130,
        'type': 'CNAME',
        'name': 'bar',
        'cname': 'example.org.',
        'ttl': 10800,
        'comment': '',
    },
]


HOSTTECH_JSON_ZONE_LIST_RESULT = {
    "data": [
        {
            "id": 42,
            "name": "example.com",
            "email": "test@example.com",
            "ttl": 10800,
            "nameserver": "ns1.hosttech.ch",
            "dnssec": False,
        },
    ],
}

HOSTTECH_JSON_ZONE_GET_RESULT = {
    "data": {
        "id": 42,
        "name": "example.com",
        "email": "test@example.com",
        "ttl": 10800,
        "nameserver": "ns1.hosttech.ch",
        "dnssec": False,
        "records": HOSTTECH_JSON_DEFAULT_ENTRIES,
    }
}

HOSTTECH_JSON_ZONE_RECORDS_GET_RESULT = {
    "data": HOSTTECH_JSON_DEFAULT_ENTRIES,
}


original_exists = os.path.exists
original_access = os.access


def exists_mock(path, exists=True):
    def exists(f):
        if to_native(f) == path:
            return exists
        return original_exists(f)

    return exists


def access_mock(path, can_access=True):
    def access(f, m, *args, **kwargs):
        if to_native(f) == path:
            return can_access
        return original_access(f, m, *args, **kwargs)

    return access


def test_inventory_file_simple(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    zone_name: example.com
    filters:
      type: A
    """)}

    open_url = OpenUrlProxy([
        OpenUrlCall('GET', 200)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
        .expect_query_values('query', 'example.com')
        .return_header('Content-Type', 'application/json')
        .result_json(HOSTTECH_JSON_ZONE_LIST_RESULT),
        OpenUrlCall('GET', 200)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/42')
        .return_header('Content-Type', 'application/json')
        .result_json(HOSTTECH_JSON_ZONE_GET_RESULT),
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert im._inventory.hosts
    assert 'example.com' in im._inventory.hosts
    assert '*.example.com' in im._inventory.hosts
    assert 'foo.example.com' not in im._inventory.hosts
    assert 'bar.example.com' not in im._inventory.hosts
    assert im._inventory.get_host('example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('*.example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('example.com').get_vars()['ansible_host'] == '1.2.3.4'
    assert im._inventory.get_host('*.example.com').get_vars()['ansible_host'] == '1.2.3.5'
    assert len(im._inventory.groups['ungrouped'].hosts) == 2
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_collision(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: "{{ 'foo' }}"
    zone_name: "{{ 'example' ~ '.com' }}"
    filters:
      type:
        - A
        - AAAA
    """)}

    open_url = OpenUrlProxy([
        OpenUrlCall('GET', 200)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
        .expect_query_values('query', 'example.com')
        .return_header('Content-Type', 'application/json')
        .result_json(HOSTTECH_JSON_ZONE_LIST_RESULT),
        OpenUrlCall('GET', 200)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/42')
        .return_header('Content-Type', 'application/json')
        .result_json(HOSTTECH_JSON_ZONE_GET_RESULT),
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert im._inventory.hosts
    assert 'example.com' in im._inventory.hosts
    assert '*.example.com' in im._inventory.hosts
    assert 'foo.example.com' in im._inventory.hosts
    assert 'bar.example.com' not in im._inventory.hosts
    assert im._inventory.get_host('example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('*.example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('foo.example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('example.com').get_vars()['ansible_host'] == '2001:1:2::3'
    assert im._inventory.get_host('*.example.com').get_vars()['ansible_host'] == '1.2.3.5'
    assert im._inventory.get_host('foo.example.com').get_vars()['ansible_host'] == '2001:1:2::4'
    assert len(im._inventory.groups['ungrouped'].hosts) == 3
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_no_filter(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    zone_id: 42
    """)}

    open_url = OpenUrlProxy([
        OpenUrlCall('GET', 200)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/42')
        .return_header('Content-Type', 'application/json')
        .result_json(HOSTTECH_JSON_ZONE_GET_RESULT),
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert im._inventory.hosts
    assert 'example.com' in im._inventory.hosts
    assert '*.example.com' in im._inventory.hosts
    assert 'foo.example.com' in im._inventory.hosts
    assert 'bar.example.com' in im._inventory.hosts
    assert im._inventory.get_host('example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('*.example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('foo.example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('bar.example.com') in im._inventory.groups['ungrouped'].hosts
    assert im._inventory.get_host('example.com').get_vars()['ansible_host'] == '2001:1:2::3'
    assert im._inventory.get_host('*.example.com').get_vars()['ansible_host'] == '1.2.3.5'
    assert im._inventory.get_host('foo.example.com').get_vars()['ansible_host'] == '2001:1:2::4'
    assert im._inventory.get_host('bar.example.com').get_vars()['ansible_host'] == 'example.org.'
    assert len(im._inventory.groups['ungrouped'].hosts) == 4
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_invalid_zone_id(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    zone_id: invalid
    """)}

    open_url = OpenUrlProxy([
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_missing_zone(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    """)}

    open_url = OpenUrlProxy([
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_zone_not_found(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    zone_id: "{{ 11 + 12 }}"
    """)}

    open_url = OpenUrlProxy([
        OpenUrlCall('GET', 404)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/23')
        .return_header('Content-Type', 'application/json')
        .result_json(dict(message="")),
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_unauthorized(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    zone_id: 23
    """)}

    open_url = OpenUrlProxy([
        OpenUrlCall('GET', 403)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/23')
        .result_json(dict(message="")),
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_file_error(mocker):
    inventory_filename = "test.hosttech_dns.yaml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    zone_id: 42
    """)}

    open_url = OpenUrlProxy([
        OpenUrlCall('GET', 500)
        .expect_header('accept', 'application/json')
        .expect_header('authorization', 'Bearer foo')
        .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/42')
        .result_json({}),
    ])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_wrong_file(mocker):
    inventory_filename = "test.hetznerdns.yml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']
    inventory_file = {inventory_filename: textwrap.dedent("""\
    ---
    plugin: community.dns.hosttech_dns_records
    hosttech_token: foo
    """)}

    open_url = OpenUrlProxy([])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename))
    mocker.patch('os.access', access_mock(inventory_filename))
    im = InventoryManager(loader=DictDataLoader(inventory_file), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0


def test_inventory_no_file(mocker):
    inventory_filename = "test.hosttech_dns.yml"
    C.INVENTORY_ENABLED = ['community.dns.hosttech_dns_records']

    open_url = OpenUrlProxy([])
    mocker.patch('ansible_collections.community.dns.plugins.module_utils.http.open_url', open_url)
    mocker.patch('ansible.inventory.manager.unfrackpath', mock_unfrackpath_noop)
    mocker.patch('os.path.exists', exists_mock(inventory_filename, False))
    mocker.patch('os.access', access_mock(inventory_filename, False))
    im = InventoryManager(loader=DictDataLoader({}), sources=inventory_filename)

    open_url.assert_is_done()

    assert not im._inventory.hosts
    assert len(im._inventory.groups['ungrouped'].hosts) == 0
    assert len(im._inventory.groups['all'].hosts) == 0



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: qos_device_interface
short_description: Resource module for Qos Device Interface
description:
- Manage operations create, update and delete of the resource Qos Device Interface.
- >
   Create qos device interface infos associate with network device id to allow the user to mark specific interfaces
   as WAN, to associate WAN interfaces with specific SP Profile and to be able to define a shaper on WAN interfaces.
- Delete all qos device interface infos associate with network device id.
- Update existing qos device interface infos associate with network device id.
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Id of the qos device info, this object holds all
      qos device interface infos associate with network device id.
    type: str
  payload:
    description: Qos Device Interface's payload.
    elements: dict
    suboptions:
      excludedInterfaces:
        description: Excluded interfaces ids.
        elements: str
        type: list
      id:
        description: Id of Qos device info.
        type: str
      name:
        description: Device name.
        type: str
      networkDeviceId:
        description: Network device id.
        type: str
      qosDeviceInterfaceInfo:
        description: Qos Device Interface's qosDeviceInterfaceInfo.
        elements: dict
        suboptions:
          dmvpnRemoteSitesBw:
            description: Dmvpn remote sites bandwidth.
            elements: int
            type: list
          instanceId:
            description: Instance id.
            type: int
          interfaceId:
            description: Interface id.
            type: str
          interfaceName:
            description: Interface name.
            type: str
          label:
            description: SP Profile name.
            type: str
          role:
            description: Interface role.
            type: str
          uploadBW:
            description: Upload bandwidth.
            type: int
        type: list
    type: list
requirements:
- dnacentersdk >= 2.5.0
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Application Policy CreateQosDeviceInterfaceInfo
  description: Complete reference of the CreateQosDeviceInterfaceInfo API.
  link: https://developer.cisco.com/docs/dna-center/#!create-qos-device-interface-info
- name: Cisco DNA Center documentation for Application Policy DeleteQosDeviceInterfaceInfo
  description: Complete reference of the DeleteQosDeviceInterfaceInfo API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-qos-device-interface-info
- name: Cisco DNA Center documentation for Application Policy UpdateQosDeviceInterfaceInfo
  description: Complete reference of the UpdateQosDeviceInterfaceInfo API.
  link: https://developer.cisco.com/docs/dna-center/#!update-qos-device-interface-info
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.create_qos_device_interface_info,
    application_policy.ApplicationPolicy.delete_qos_device_interface_info,
    application_policy.ApplicationPolicy.update_qos_device_interface_info,

  - Paths used are
    post /dna/intent/api/v1/qos-device-interface-info,
    delete /dna/intent/api/v1/qos-device-interface-info/{id},
    put /dna/intent/api/v1/qos-device-interface-info,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.qos_device_interface:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - excludedInterfaces:
      - string
      id: string
      name: string
      networkDeviceId: string
      qosDeviceInterfaceInfo:
      - dmvpnRemoteSitesBw:
        - 0
        instanceId: 0
        interfaceId: string
        interfaceName: string
        label: string
        role: string
        uploadBW: 0

- name: Create
  cisco.dnac.qos_device_interface:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - excludedInterfaces:
      - string
      name: string
      networkDeviceId: string
      qosDeviceInterfaceInfo:
      - dmvpnRemoteSitesBw:
        - 0
        interfaceId: string
        interfaceName: string
        label: string
        role: string
        uploadBW: 0

- name: Delete by id
  cisco.dnac.qos_device_interface:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: mso_schema_site_anp_epg
short_description: Manage site-local Endpoint Groups (EPGs) in schema template
description:
- Manage site-local EPGs in schema template on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: yes
  site:
    description:
    - The name of the site.
    type: str
    required: yes
  template:
    description:
    - The name of the template.
    type: str
    required: yes
  anp:
    description:
    - The name of the ANP.
    type: str
    required: yes
  epg:
    description:
    - The name of the EPG to manage.
    type: str
    aliases: [ name ]
  private_link_label:
    description:
    - The private link label used to represent this subnet.
    - This parameter is available for MSO version greater than 3.3.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_site_anp
- module: cisco.mso.mso_schema_site_anp_epg_subnet
- module: cisco.mso.mso_schema_template_anp_epg
extends_documentation_fragment: cisco.mso.modules
'''

EXAMPLES = r'''
- name: Add a new site EPG
  cisco.mso.mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: present
  delegate_to: localhost

- name: Remove a site EPG
  cisco.mso.mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: absent
  delegate_to: localhost

- name: Query a specific site EPGs
  cisco.mso.mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all site EPGs
  cisco.mso.mso_schema_site_anp_epg:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type='str', required=True),
        site=dict(type='str', required=True),
        template=dict(type='str', required=True),
        anp=dict(type='str', required=True),
        epg=dict(type='str', aliases=['name']),  # This parameter is not required for querying all objects
        private_link_label=dict(type='str'),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['epg']],
            ['state', 'present', ['epg']],
        ],
    )

    schema = module.params.get('schema')
    site = module.params.get('site')
    template = module.params.get('template').replace(' ', '')
    anp = module.params.get('anp')
    epg = module.params.get('epg')
    private_link_label = module.params.get('private_link_label')
    state = module.params.get('state')

    mso = MSOModule(module)

    # Get schema objects
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get template
    templates = [t.get('name') for t in schema_obj.get('templates')]
    if template not in templates:
        mso.fail_json(msg="Provided template '{0}' does not exist. Existing templates: {1}".format(template, ', '.join(templates)))
    template_idx = templates.index(template)

    # Get site
    site_id = mso.lookup_site(site)

    # Get site_idx
    if 'sites' not in schema_obj:
        mso.fail_json(msg="No site associated with template '{0}'. Associate the site with the template using mso_schema_site.".format(template))
    sites = [(s.get('siteId'), s.get('templateName')) for s in schema_obj.get('sites')]
    sites_list = [s.get('siteId') + '/' + s.get('templateName') for s in schema_obj.get('sites')]
    if (site_id, template) not in sites:
        mso.fail_json(msg="Provided site/siteId/template '{0}/{1}/{2}' does not exist. "
                          "Existing siteIds/templates: {3}".format(site, site_id, template, ', '.join(sites_list)))

    # Schema-access uses indexes
    site_idx = sites.index((site_id, template))
    # Path-based access uses site_id-template
    site_template = '{0}-{1}'.format(site_id, template)

    payload = {}
    ops = []
    op_path = ''

    # Get ANP
    anp_ref = mso.anp_ref(schema_id=schema_id, template=template, anp=anp)
    anps = [a.get('anpRef') for a in schema_obj['sites'][site_idx]['anps']]
    anps_in_temp = [a.get('name') for a in schema_obj['templates'][template_idx]['anps']]
    if anp not in anps_in_temp:
        mso.fail_json(msg="Provided anp '{0}' does not exist. Existing anps: {1}".format(anp, ', '.join(anps_in_temp)))
    else:
        # Get anp index at template level
        template_anp_idx = anps_in_temp.index(anp)

    # If anp not at site level but exists at template level
    if anp_ref not in anps:
        op_path = '/sites/{0}/anps'.format(site_template)
        payload = dict(
            anpRef=dict(
                schemaId=schema_id,
                templateName=template,
                anpName=anp,
            ),
        )
    else:
        # Get anp index at site level
        anp_idx = anps.index(anp_ref)

    if epg is not None:
        # Get EPG
        epg_ref = mso.epg_ref(schema_id=schema_id, template=template, anp=anp, epg=epg)
        new_epg = dict(
            epgRef=dict(
                schemaId=schema_id,
                templateName=template,
                anpName=anp,
                epgName=epg,
            )
        )

        # If anp exists at site level
        if 'anpRef' not in payload:
            epgs = [e.get('epgRef') for e in schema_obj['sites'][site_idx]['anps'][anp_idx]['epgs']]

        # If anp already at site level AND if epg not at site level (or) anp not at site level?
        if ('anpRef' not in payload and epg_ref not in epgs) or 'anpRef' in payload:
            epgs_in_temp = [e.get('name') for e in schema_obj['templates'][template_idx]['anps'][template_anp_idx]['epgs']]

            # If EPG not at template level - Fail
            if epg not in epgs_in_temp:
                mso.fail_json(msg="Provided EPG '{0}' does not exist. Existing EPGs: {1}".format(epg, ', '.join(epgs_in_temp)))

            # EPG at template level but not at site level. Create payload at site level for EPG
            else:
                # If anp not in payload then, anp already exists at site level. New payload will only have new EPG payload
                if 'anpRef' not in payload:
                    op_path = '/sites/{0}/anps/{1}/epgs'.format(site_template, anp)
                    payload = new_epg
                else:
                    # If anp in payload, anp exists at site level. Update payload with EPG payload
                    payload['epgs'] = [new_epg]

        # Get index of EPG at site level
        else:
            epg_idx = epgs.index(epg_ref)
            epg_path = '/sites/{0}/anps/{1}/epgs/{2}'.format(site_template, anp, epg)
            mso.existing = schema_obj.get('sites')[site_idx]['anps'][anp_idx]['epgs'][epg_idx]
            payload = new_epg

    ops = []

    if state == 'query':
        if anp_ref not in anps:
            mso.fail_json(msg="Provided anp '{0}' does not exist at site level.".format(anp))
        if epg is None:
            mso.existing = schema_obj.get('sites')[site_idx]['anps'][anp_idx]['epgs']
        elif not mso.existing:
            mso.fail_json(msg="EPG '{epg}' not found".format(epg=epg))
        mso.exit_json()

    mso.previous = mso.existing

    if state == 'absent':
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op='remove', path=epg_path))

    elif state == 'present':

        if private_link_label is not None:
            payload['privateLinkLabel'] = dict(name=private_link_label)

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op='replace', path=epg_path, value=mso.sent))
        else:
            ops.append(dict(op='add', path=op_path + '/-', value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(schema_path, method='PATCH', data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest
try:
    from botocore.exceptions import ClientError
except ImportError:
    pass

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import HAS_BOTO3
from ansible_collections.community.aws.tests.unit.compat.mock import call
from ansible_collections.community.aws.tests.unit.compat.mock import patch
from ansible_collections.community.aws.tests.unit.plugins.modules.utils import AnsibleExitJson
from ansible_collections.community.aws.tests.unit.plugins.modules.utils import AnsibleFailJson
from ansible_collections.community.aws.tests.unit.plugins.modules.utils import ModuleTestCase
from ansible_collections.community.aws.tests.unit.plugins.modules.utils import set_module_args

from ansible_collections.community.aws.plugins.modules import aws_direct_connect_confirm_connection

if not HAS_BOTO3:
    pytestmark = pytest.mark.skip("test_aws_direct_connect_confirm_connection.py requires the `boto3` and `botocore` modules")


@patch('ansible_collections.amazon.aws.plugins.module_utils.core.HAS_BOTO3', new=True)
@patch.object(aws_direct_connect_confirm_connection.AnsibleAWSModule, "client")
class TestAWSDirectConnectConfirmConnection(ModuleTestCase):
    def test_missing_required_parameters(self, *args):
        set_module_args({})
        with self.assertRaises(AnsibleFailJson) as exec_info:
            aws_direct_connect_confirm_connection.main()

        result = exec_info.exception.args[0]
        assert result["failed"] is True
        assert "name" in result["msg"]
        assert "connection_id" in result["msg"]

    def test_get_by_connection_id(self, mock_client):
        mock_client.return_value.describe_connections.return_value = {
            "connections": [
                {
                    "connectionState": "requested",
                    "connectionId": "dxcon-fgq9rgot",
                    "location": "EqSe2",
                    "connectionName": "ansible-test-connection",
                    "bandwidth": "1Gbps",
                    "ownerAccount": "448830907657",
                    "region": "us-west-2"
                }
            ]
        }
        set_module_args({
            "connection_id": "dxcon-fgq9rgot"
        })
        with self.assertRaises(AnsibleExitJson) as exec_info:
            aws_direct_connect_confirm_connection.main()

        result = exec_info.exception.args[0]
        assert result["changed"] is False
        assert result["connection_state"] == "requested"
        mock_client.return_value.describe_connections.assert_has_calls([
            call(connectionId="dxcon-fgq9rgot")
        ])
        mock_client.return_value.confirm_connection.assert_not_called()

    def test_get_by_name(self, mock_client):
        mock_client.return_value.describe_connections.return_value = {
            "connections": [
                {
                    "connectionState": "requested",
                    "connectionId": "dxcon-fgq9rgot",
                    "location": "EqSe2",
                    "connectionName": "ansible-test-connection",
                    "bandwidth": "1Gbps",
                    "ownerAccount": "448830907657",
                    "region": "us-west-2"
                }
            ]
        }
        set_module_args({
            "name": "ansible-test-connection"
        })
        with self.assertRaises(AnsibleExitJson) as exec_info:
            aws_direct_connect_confirm_connection.main()

        result = exec_info.exception.args[0]
        assert result["changed"] is False
        assert result["connection_state"] == "requested"
        mock_client.return_value.describe_connections.assert_has_calls([
            call(),
            call(connectionId="dxcon-fgq9rgot")
        ])
        mock_client.return_value.confirm_connection.assert_not_called()

    def test_missing_connection_id(self, mock_client):
        mock_client.return_value.describe_connections.side_effect = ClientError(
            {'Error': {'Code': 'ResourceNotFoundException'}}, 'DescribeConnection')
        set_module_args({
            "connection_id": "dxcon-aaaabbbb"
        })
        with self.assertRaises(AnsibleFailJson) as exec_info:
            aws_direct_connect_confirm_connection.main()

        result = exec_info.exception.args[0]
        assert result["failed"] is True
        mock_client.return_value.describe_connections.assert_has_calls([
            call(connectionId="dxcon-aaaabbbb")
        ])

    def test_missing_name(self, mock_client):
        mock_client.return_value.describe_connections.return_value = {
            "connections": [
                {
                    "connectionState": "requested",
                    "connectionId": "dxcon-fgq9rgot",
                    "location": "EqSe2",
                    "connectionName": "ansible-test-connection",
                    "bandwidth": "1Gbps",
                    "ownerAccount": "448830907657",
                    "region": "us-west-2"
                }
            ]
        }
        set_module_args({
            "name": "foobar"
        })
        with self.assertRaises(AnsibleFailJson) as exec_info:
            aws_direct_connect_confirm_connection.main()

        result = exec_info.exception.args[0]
        assert result["failed"] is True
        mock_client.return_value.describe_connections.assert_has_calls([
            call()
        ])

    def test_confirm(self, mock_client):
        mock_client.return_value.describe_connections.return_value = {
            "connections": [
                {
                    "connectionState": "ordering",
                    "connectionId": "dxcon-fgq9rgot",
                    "location": "EqSe2",
                    "connectionName": "ansible-test-connection",
                    "bandwidth": "1Gbps",
                    "ownerAccount": "448830907657",
                    "region": "us-west-2"
                }
            ]
        }
        mock_client.return_value.confirm_connection.return_value = [{}]
        set_module_args({
            "connection_id": "dxcon-fgq9rgot"
        })
        with self.assertRaises(AnsibleExitJson) as exec_info:
            aws_direct_connect_confirm_connection.main()

        result = exec_info.exception.args[0]
        assert result["changed"] is True
        mock_client.return_value.describe_connections.assert_has_calls([
            call(connectionId="dxcon-fgq9rgot"),
            call(connectionId="dxcon-fgq9rgot"),
            call(connectionId="dxcon-fgq9rgot")
        ])
        mock_client.return_value.confirm_connection.assert_called_once_with(connectionId="dxcon-fgq9rgot")



# -*- coding: utf-8 -*-
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type


import json

import pytest

from ansible_collections.community.internal_test_tools.tests.unit.compat.mock import MagicMock, patch

from ansible_collections.community.dns.plugins.module_utils.record import (
    DNSRecord,
)

from ansible_collections.community.dns.plugins.module_utils.zone_record_api import (
    DNSAPIError,
)

from ansible_collections.community.dns.plugins.module_utils.hosttech.json_api import (
    _create_record_from_json,
    _record_to_json,
    HostTechJSONAPI,
)


# The example JSONs for all record types are taken from https://api.ns1.hosttech.eu/api/documentation/

def test_AAAA():
    data = {
        "id": 11,
        "type": "AAAA",
        "name": "www",
        "ipv6": "2001:db8:1234::1",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 11
    assert record.type == 'AAAA'
    assert record.prefix == 'www'
    assert record.target == '2001:db8:1234::1'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data


def test_A():
    data = {
        "id": 10,
        "type": "A",
        "name": "www",
        "ipv4": "1.2.3.4",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 10
    assert record.type == 'A'
    assert record.prefix == 'www'
    assert record.target == '1.2.3.4'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data


def test_CAA():
    data = {
        "id": 12,
        "type": "CAA",
        "name": "",
        "flag": "0",
        "tag": "issue",
        "value": "letsencrypt.org",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 12
    assert record.type == 'CAA'
    assert record.prefix is None
    assert record.target == '0 issue "letsencrypt.org"'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data

    # We also accept versions without quotes:
    record.target = '0 issue letsencrypt.org'
    assert _record_to_json(record, include_id=True) == data

    record.target = '0\tissue "letsencrypt.org"'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith('Cannot split CAA record "0\tissue "letsencrypt.org"" into flag, tag and value: ')


def test_CNAME():
    data = {
        "id": 13,
        "type": "CNAME",
        "name": "www",
        "cname": "site.example.com",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 13
    assert record.type == 'CNAME'
    assert record.prefix == 'www'
    assert record.target == 'site.example.com'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data


def test_MX():
    data = {
        "id": 14,
        "type": "MX",
        "ownername": "",
        "name": "mail.example.com",
        "pref": 10,
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 14
    assert record.type == 'MX'
    assert record.prefix is None
    assert record.target == '10 mail.example.com'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data

    record.target = 'mail.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith('Cannot split MX record "mail.example.com" into integer preference and name: ')

    record.target = 'x mail.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith('Cannot split MX record "x mail.example.com" into integer preference and name: ')


def test_NS():
    # WARNING: as opposed to documented on https://api.ns1.hosttech.eu/api/documentation/,
    #          NS records use 'targetname' and not 'name'!
    data = {
        "id": 14,
        "type": "NS",
        "ownername": "sub",
        "targetname": "ns1.example.com",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 14
    assert record.type == 'NS'
    assert record.prefix == 'sub'
    assert record.target == 'ns1.example.com'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data


def test_PTR():
    data = {
        "id": 15,
        "type": "PTR",
        "origin": "4.3.2.1",
        "name": "smtp.example.com",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 15
    assert record.type == 'PTR'
    assert record.prefix is None
    assert record.target == '4.3.2.1 smtp.example.com'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data

    record.target = 'smtp.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith('Cannot split PTR record "smtp.example.com" into origin and name: ')


def test_SRV():
    data = {
        "id": 16,
        "type": "SRV",
        "service": "_autodiscover._tcp",
        "priority": 0,
        "weight": 1,
        "port": 443,
        "target": "exchange.example.com",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 16
    assert record.type == 'SRV'
    assert record.prefix == '_autodiscover._tcp'
    assert record.target == '0 1 443 exchange.example.com'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data

    record.target = '1 443 exchange.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith(
        'Cannot split SRV record "1 443 exchange.example.com" into integer priority, integer weight, integer port and target: ')

    record.target = 'x 1 443 exchange.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith(
        'Cannot split SRV record "x 1 443 exchange.example.com" into integer priority, integer weight, integer port and target: ')

    record.target = '0 x 443 exchange.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith(
        'Cannot split SRV record "0 x 443 exchange.example.com" into integer priority, integer weight, integer port and target: ')

    record.target = '0 1 x exchange.example.com'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0].startswith(
        'Cannot split SRV record "0 1 x exchange.example.com" into integer priority, integer weight, integer port and target: ')


def test_TXT():
    data = {
        "id": 17,
        "type": "TXT",
        "name": "",
        "text": "v=spf1 ip4:1.2.3.4/32 -all",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 17
    assert record.type == 'TXT'
    assert record.prefix is None
    assert record.target == 'v=spf1 ip4:1.2.3.4/32 -all'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data


def test_TLSA():
    data = {
        "id": 17,
        "type": "TLSA",
        "name": "",
        "text": "0 0 1 d2abde240d7cd3ee6b4b28c54df034b97983a1d16e8a410e4561cb106618e971",
        "ttl": 3600,
        "comment": "my first record",
    }
    record = _create_record_from_json(data)
    assert record.id == 17
    assert record.type == 'TLSA'
    assert record.prefix is None
    assert record.target == '0 0 1 d2abde240d7cd3ee6b4b28c54df034b97983a1d16e8a410e4561cb106618e971'
    assert record.ttl == 3600
    assert record.extra == {
        'comment': 'my first record',
    }
    assert _record_to_json(record, include_id=True) == data


def test_unknown_records():
    data = {
        "id": 17,
        "type": "unknown",
        "name": "",
        "ttl": 3600,
        "comment": "my first record",
    }
    with pytest.raises(DNSAPIError) as exc:
        _create_record_from_json(data)
    assert exc.value.args[0] == 'Cannot parse unknown record type: unknown'

    record = DNSRecord()
    record.type = 'unknown'
    with pytest.raises(DNSAPIError) as exc:
        _record_to_json(record)
    assert exc.value.args[0] == 'Cannot serialize unknown record type: unknown'


def test_list_pagination():
    def get_1(url, query=None, must_have_content=True, expected=None):
        assert url == 'https://example.com'
        assert must_have_content is True
        assert expected == [200]
        assert query is not None
        assert len(query) == 2
        assert query['limit'] == 1
        assert query['offset'] in [0, 1, 2]
        if query['offset'] < 2:
            return {'data': [query['offset']]}, {}
        else:
            return {'data': []}, {}

    def get_2(url, query=None, must_have_content=True, expected=None):
        assert url == 'https://example.com'
        assert must_have_content is True
        assert expected == [200]
        assert query is not None
        assert len(query) == 3
        assert query['foo'] == 'bar'
        assert query['limit'] == 2
        assert query['offset'] in [0, 2]
        if query['offset'] < 2:
            return {'data': ['bar', 'baz']}, {}
        else:
            return {'data': ['foo']}, {}

    api = HostTechJSONAPI(MagicMock(), '123')

    api._get = MagicMock(side_effect=get_1)
    result = api._list_pagination('https://example.com', block_size=1)
    assert result == [0, 1]

    api._get = MagicMock(side_effect=get_2)
    result = api._list_pagination('https://example.com', query=dict(foo='bar'), block_size=2)
    assert result == ['bar', 'baz', 'foo']


def test_update_id_missing():
    api = HostTechJSONAPI(MagicMock(), '123')
    with pytest.raises(DNSAPIError) as exc:
        api.update_record(1, DNSRecord())
    assert exc.value.args[0] == 'Need record ID to update record!'


def test_update_id_delete():
    api = HostTechJSONAPI(MagicMock(), '123')
    with pytest.raises(DNSAPIError) as exc:
        api.delete_record(1, DNSRecord())
    assert exc.value.args[0] == 'Need record ID to delete record!'


def test_extract_error_message():
    api = HostTechJSONAPI(MagicMock(), '123')
    assert api._extract_error_message(None) == ''
    assert api._extract_error_message('foo') == ' with data: foo'
    assert api._extract_error_message(dict()) == ' with data: {}'
    assert api._extract_error_message(dict(message='')) == " with data: {'message': ''}"
    assert api._extract_error_message(dict(message='foo')) == ' with message "foo"'
    assert api._extract_error_message(dict(message='foo', errors='')) == ' with message "foo"'
    assert api._extract_error_message(dict(message='foo', errors=dict())) == ' with message "foo"'
    assert api._extract_error_message(dict(message='foo', errors=dict(bar='baz'))) == ' with message "foo" (field "bar": baz)'
    assert api._extract_error_message(dict(errors=dict(bar=['baz', 'bam'], arf='fra'))) == ' (field "arf": fra) (field "bar": baz; bam)'



#
# (c) 2022, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from textwrap import dedent
from ansible_collections.cisco.ios.tests.unit.compat.mock import patch
from ansible_collections.cisco.ios.plugins.modules import ios_interfaces
from ansible_collections.cisco.ios.tests.unit.modules.utils import (
    set_module_args,
)
from .ios_module import TestIosModule


class TestIosInterfacesModule(TestIosModule):
    module = ios_interfaces

    def setUp(self):
        super(TestIosInterfacesModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_resource_connection_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base."
            "get_resource_connection"
        )
        self.get_resource_connection_config = (
            self.mock_get_resource_connection_config.start()
        )

        self.mock_get_resource_connection_facts = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base."
            "get_resource_connection"
        )
        self.get_resource_connection_facts = (
            self.mock_get_resource_connection_facts.start()
        )

        self.mock_edit_config = patch(
            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.providers.CliProvider.edit_config"
        )
        self.edit_config = self.mock_edit_config.start()

        self.mock_execute_show_command = patch(
            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.interfaces.interfaces."
            "InterfacesFacts.get_interfaces_data"
        )
        self.execute_show_command = self.mock_execute_show_command.start()

    def tearDown(self):
        super(TestIosInterfacesModule, self).tearDown()
        self.mock_get_resource_connection_config.stop()
        self.mock_get_resource_connection_facts.stop()
        self.mock_edit_config.stop()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_execute_show_command.stop()

    def test_ios_interfaces_merged(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            {
                "config": [
                    {
                        "description": "This interface should be disabled",
                        "enabled": True,
                        "name": "GigabitEthernet1",
                    },
                    {
                        "description": "This interface should be enabled",
                        "enabled": False,
                        "name": "GigabitEthernet2",
                    },
                ],
                "state": "merged",
            }
        )
        commands = [
            "interface GigabitEthernet1",
            "description This interface should be disabled",
            "interface GigabitEthernet2",
            "description This interface should be enabled",
            "shutdown",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], commands)

    def test_ios_interfaces_merged_idempotent(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            {
                "config": [
                    {
                        "description": "Ansible UT interface 1",
                        "name": "GigabitEthernet1",
                    },
                    {
                        "description": "Ansible UT interface 2",
                        "name": "GigabitEthernet2",
                        "speed": 1000,
                        "mtu": 1500,
                    },
                ],
                "state": "merged",
            }
        )
        self.execute_module(changed=False, commands=[])

    def test_ios_interfaces_replaced(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            {
                "config": [
                    {
                        "description": "Ansible UT interface 1",
                        "name": "GigabitEthernet1",
                    },
                    {"name": "GigabitEthernet2", "speed": 1200, "mtu": 1800},
                ],
                "state": "replaced",
            }
        )
        commands = [
            "interface GigabitEthernet2",
            "no description Ansible UT interface 2",
            "speed 1200",
            "mtu 1800",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], commands)

    def test_ios_interfaces_replaced_idempotent(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            {
                "config": [
                    {
                        "description": "Ansible UT interface 1",
                        "name": "GigabitEthernet1",
                    },
                    {
                        "description": "Ansible UT interface 2",
                        "name": "GigabitEthernet2",
                        "speed": 1000,
                        "mtu": 1500,
                    },
                ],
                "state": "replaced",
            }
        )
        self.execute_module(changed=False, commands=[])

    def test_ios_interfaces_overridden(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            {
                "config": [
                    {
                        "description": "Ansible UT interface try 1",
                        "speed": 1000,
                        "name": "GigabitEthernet1",
                    },
                    {
                        "description": "Ansible UT interface 2",
                        "name": "GigabitEthernet3",
                        "speed": 1000,
                        "mtu": 1500,
                    },
                ],
                "state": "overridden",
            }
        )

        commands = [
            "interface GigabitEthernet2",
            "no description Ansible UT interface 2",
            "no speed 1000",
            "no mtu 1500",
            "shutdown",
            "interface GigabitEthernet4",
            "no description Ansible UT interface 4",
            "interface GigabitEthernet5",
            "no description Ansible UT interface 5",
            "no duplex full",
            "shutdown",
            "interface GigabitEthernet1",
            "description Ansible UT interface try 1",
            "speed 1000",
            "interface GigabitEthernet3",
            "description Ansible UT interface 2",
            "speed 1000",
            "mtu 1500",
            "no duplex auto",
            "no shutdown",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], commands)

    def test_ios_interfaces_deleted(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            dict(config=[dict(name="GigabitEthernet1")], state="deleted")
        )
        commands = [
            "interface GigabitEthernet1",
            "no description Ansible UT interface 1",
            "shutdown",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], commands)

    def test_ios_interfaces_purged(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(
            dict(
                config=[
                    {
                        "name": "GigabitEthernet2",
                        "description": "Ansible UT interface 2",
                        "speed": "1000",
                        "mtu": 1500,
                        "enabled": True,
                    },
                    {
                        "name": "GigabitEthernet3",
                        "description": "Ansible UT interface 3",
                        "enabled": True,
                        "duplex": "auto",
                    },
                ],
                state="purged",
            )
        )
        commands = [
            "no interface GigabitEthernet2",
            "no interface GigabitEthernet3",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], commands)

    def test_ios_interfaces_purged_extreme(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
             no shutdown
             ip address dhcp
             negotiation auto
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
             speed 1000
             mtu 1500
             no negotiation auto
            interface GigabitEthernet3
             description Ansible UT interface 3
             no ip address
             shutdown
             duplex auto
             negotiation auto
             channel-group 10 mode active
            interface GigabitEthernet4
             description Ansible UT interface 4
             no ip address
             shutdown
             negotiation auto
            interface GigabitEthernet5
             description Ansible UT interface 5
             no ip address
             duplex full
             negotiation auto
             ipv6 dhcp server
            """
        )
        set_module_args(dict(config=[], state="purged"))
        commands = [
            "no interface GigabitEthernet1",
            "no interface GigabitEthernet2",
            "no interface GigabitEthernet3",
            "no interface GigabitEthernet4",
            "no interface GigabitEthernet5",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], commands)

    def test_ios_interfaces_parsed(self):
        set_module_args(
            dict(
                running_config=dedent(
                    """\
                    interface GigabitEthernet1
                     description Ansible UT interface 1
                     no shutdown
                     ip address dhcp
                     negotiation auto
                    interface GigabitEthernet2
                     description Ansible UT interface 2
                     ip address dhcp
                     speed 1000
                     mtu 1500
                     no negotiation auto
                    interface GigabitEthernet3
                     description Ansible UT interface 3
                     no ip address
                     shutdown
                     duplex auto
                     negotiation auto
                     channel-group 10 mode active
                    interface GigabitEthernet4
                     description Ansible UT interface 4
                     no ip address
                     shutdown
                     negotiation auto
                    interface GigabitEthernet5
                     description Ansible UT interface 5
                     no ip address
                     duplex full
                     negotiation auto
                     ipv6 dhcp server
                    """
                ),
                state="parsed",
            )
        )
        result = self.execute_module(changed=False)
        parsed_list = [
            {
                "name": "GigabitEthernet1",
                "description": "Ansible UT interface 1",
                "enabled": True,
            },
            {
                "name": "GigabitEthernet2",
                "description": "Ansible UT interface 2",
                "speed": "1000",
                "mtu": 1500,
                "enabled": True,
            },
            {
                "name": "GigabitEthernet3",
                "description": "Ansible UT interface 3",
                "enabled": False,
                "duplex": "auto",
            },
            {
                "name": "GigabitEthernet4",
                "description": "Ansible UT interface 4",
                "enabled": False,
            },
            {
                "name": "GigabitEthernet5",
                "description": "Ansible UT interface 5",
                "duplex": "full",
                "enabled": True,
            },
        ]
        self.assertEqual(parsed_list, result["parsed"])

    def test_ios_interfaces_rendered(self):
        self.execute_show_command.return_value = dedent(
            """\
            interface GigabitEthernet1
             description Ansible UT interface 1
            interface GigabitEthernet2
             description Ansible UT interface 2
             ip address dhcp
            interface GigabitEthernet3
             no ip address
            interface GigabitEthernet4
             negotiation auto
            interface GigabitEthernet5
             ipv6 dhcp server
            """
        )
        set_module_args(
            dict(
                config=[
                    {
                        "name": "GigabitEthernet1",
                        "description": "Ansible UT interface 1",
                        "enabled": False,
                    },
                    {
                        "name": "GigabitEthernet2",
                        "description": "Ansible UT interface 2",
                        "speed": "1000",
                        "mtu": 1500,
                        "enabled": True,
                    },
                    {
                        "name": "GigabitEthernet3",
                        "description": "Ansible UT interface 3",
                        "enabled": True,
                        "duplex": "auto",
                    },
                    {
                        "name": "GigabitEthernet4",
                        "description": "Ansible UT interface 4",
                        "enabled": True,
                    },
                    {
                        "name": "GigabitEthernet5",
                        "description": "Ansible UT interface 5",
                        "duplex": "full",
                        "enabled": True,
                    },
                ],
                state="rendered",
            )
        )

        commands = [
            "interface GigabitEthernet1",
            "description Ansible UT interface 1",
            "shutdown",
            "interface GigabitEthernet2",
            "description Ansible UT interface 2",
            "speed 1000",
            "mtu 1500",
            "no shutdown",
            "interface GigabitEthernet3",
            "description Ansible UT interface 3",
            "duplex auto",
            "no shutdown",
            "interface GigabitEthernet4",
            "description Ansible UT interface 4",
            "no shutdown",
            "interface GigabitEthernet5",
            "description Ansible UT interface 5",
            "duplex full",
            "no shutdown",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["rendered"]), sorted(commands))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tag_member_info
short_description: Information module for Tag Member
description:
- Get all Tag Member.
- Returns tag members specified by id.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id path parameter. Tag ID.
    type: str
  memberType:
    description:
    - >
      MemberType query parameter. Entity type of the member. Possible values can be retrieved by using
      /tag/member/type API.
    type: str
  offset:
    description:
    - Offset query parameter. Used for pagination. It indicates the starting row number out of available member records.
    type: str
  limit:
    description:
    - Limit query parameter. Used to Number of maximum members to return in the result.
    type: str
  memberAssociationType:
    description:
    - >
      MemberAssociationType query parameter. Indicates how the member is associated with the tag. Possible values
      and description. 1) DYNAMIC The member is associated to the tag through rules. 2) STATIC – The member is
      associated to the tag manually. 3) MIXED – The member is associated manually and also satisfies the rule
      defined for the tag.
    type: str
  level:
    description:
    - Level query parameter.
    type: str
requirements:
- dnacentersdk >= 2.5.0
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Tag GetTagMembersById
  description: Complete reference of the GetTagMembersById API.
  link: https://developer.cisco.com/docs/dna-center/#!get-tag-members-by-id
notes:
  - SDK Method used are
    tag.Tag.get_tag_members_by_id,

  - Paths used are
    get /dna/intent/api/v1/tag/{id}/member,

"""

EXAMPLES = r"""
- name: Get all Tag Member
  cisco.dnac.tag_member_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    memberType: string
    offset: string
    limit: string
    memberAssociationType: string
    level: string
    id: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": [
        {
          "instanceUuid": "string"
        }
      ]
    }
"""



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import sys
import pytest

from ansible_collections.community.internal_test_tools.tests.unit.compat.mock import MagicMock

lxmletree = pytest.importorskip("lxml.etree")

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.dns.plugins.module_utils.wsdl import (
    Parser,
    Composer,
)


def test_composer_generation():
    composer = Composer(MagicMock(), api='https://example.com/api')
    composer.add_simple_command(
        'test',
        int_value=42,
        str_value='bar',
        list_value=[1, 2, 3],
        dict_value={
            'hello': 'world',
            'list': [2, 3, 5, 7],
        }
    )
    command = to_native(lxmletree.tostring(composer._root, pretty_print=True)).splitlines()

    print(command)

    expected_lines = [
        '  <SOAP-ENV:Header/>',
        '  <SOAP-ENV:Body>',
        '    <ns0:test xmlns:ns0="https://example.com/api">',
        '      <int_value xsi:type="xsd:int">42</int_value>',
        '      <str_value xsi:type="xsd:string">bar</str_value>',
        '      <list_value xsi:type="SOAP-ENC:Array">',
        '        <item xsi:type="xsd:int">1</item>',
        '        <item xsi:type="xsd:int">2</item>',
        '        <item xsi:type="xsd:int">3</item>',
        '      </list_value>',
        '      <dict_value xmlns:ns0="http://xml.apache.org/xml-soap" xsi:type="ns0:Map">',
        '        <item>',
        '          <key xsi:type="xsd:string">hello</key>',
        '          <value xsi:type="xsd:string">world</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">list</key>',
        '          <value xsi:type="SOAP-ENC:Array">',
        '            <item xsi:type="xsd:int">2</item>',
        '            <item xsi:type="xsd:int">3</item>',
        '            <item xsi:type="xsd:int">5</item>',
        '            <item xsi:type="xsd:int">7</item>',
        '          </value>',
        '        </item>',
        '      </dict_value>',
        '    </ns0:test>',
        '  </SOAP-ENV:Body>',
        '</SOAP-ENV:Envelope>',
    ]

    if sys.version_info < (3, 7):
        assert sorted(command[1:]) == sorted(expected_lines)
    else:
        assert command[1:] == expected_lines

    for part in [
            '<SOAP-ENV:Envelope',
            ' xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"',
            ' xmlns:xsd="http://www.w3.org/2001/XMLSchema"',
            ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"',
            ' xmlns:ns2="auth"',
            ' xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"',
            ' SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"',
    ]:
        assert part in command[0]


def test_parsing():
    input = '\n'.join([
        '<?xml version="1.0" encoding="UTF-8"?>',
        ''.join([
            '<SOAP-ENV:Envelope',
            ' xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"',
            ' xmlns:ns1="https://example.com/api"',
            ' xmlns:xsd="http://www.w3.org/2001/XMLSchema"',
            ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"',
            ' xmlns:ns2="http://xml.apache.org/xml-soap"',
            ' xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"',
            ' SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"',
            '>',
        ]),
        '  <SOAP-ENV:Header>',
        '    <ns1:authenticateResponse>',
        '      <return xsi:type="xsd:boolean">true</return>',
        '    </ns1:authenticateResponse>',
        '  </SOAP-ENV:Header>',
        '  <SOAP-ENV:Body>',
        '    <ns1:getZoneResponse>',
        '      <return xsi:type="ns2:Map">',
        '        <item>',
        '          <key xsi:type="xsd:string">id</key>',
        '          <value xsi:type="xsd:int">1</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">user</key>',
        '          <value xsi:type="xsd:int">2</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">name</key>',
        '          <value xsi:type="xsd:string">example.com</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">email</key>',
        '          <value xsi:type="xsd:string">info@example.com</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">ttl</key>',
        '          <value xsi:type="xsd:int">10800</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">nameserver</key>',
        '          <value xsi:type="xsd:string">ns1.hostserv.eu</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">serial</key>',
        '          <value xsi:type="xsd:string">1234567890</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">serialLastUpdate</key>',
        '          <value xsi:type="xsd:int">0</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">refresh</key>',
        '          <value xsi:type="xsd:int">7200</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">retry</key>',
        '          <value xsi:type="xsd:int">120</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">expire</key>',
        '          <value xsi:type="xsd:int">1234567</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">template</key>',
        '          <value xsi:nil="true"/>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">ns3</key>',
        '          <value xsi:type="xsd:int">1</value>',
        '        </item>',
        '        <item>',
        '          <key xsi:type="xsd:string">records</key>',
        '          <value SOAP-ENC:arrayType="ns2:Map[2]" xsi:type="SOAP-ENC:Array">',
        '            <item xsi:type="ns2:Map">',
        '              <item>',
        '                <key xsi:type="xsd:string">id</key>',
        '                <value xsi:type="xsd:int">3</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">zone</key>',
        '                <value xsi:type="xsd:int">4</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">type</key>',
        '                <value xsi:type="xsd:string">A</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">prefix</key>',
        '                <value xsi:type="xsd:string"></value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">target</key>',
        '                <value xsi:type="xsd:string">1.2.3.4</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">ttl</key>',
        '                <value xsi:type="xsd:int">3600</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">comment</key>',
        '                <value xsi:nil="true"/>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">priority</key>',
        '                <value xsi:nil="true"/>',
        '              </item>',
        '            </item>',
        '            <item xsi:type="ns2:Map">',
        '              <item>',
        '                <key xsi:type="xsd:string">id</key>',
        '                <value xsi:type="xsd:int">5</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">zone</key>',
        '                <value xsi:type="xsd:int">4</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">type</key>',
        '                <value xsi:type="xsd:string">A</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">prefix</key>',
        '                <value xsi:type="xsd:string">*</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">target</key>',
        '                <value xsi:type="xsd:string">1.2.3.5</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">ttl</key>',
        '                <value xsi:type="xsd:int">3600</value>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">comment</key>',
        '                <value xsi:nil="true"/>',
        '              </item>',
        '              <item>',
        '                <key xsi:type="xsd:string">priority</key>',
        '                <value xsi:nil="true"/>',
        '              </item>',
        '            </item>',
        '          </value>',
        '        </item>',
        '      </return>',
        '    </ns1:getZoneResponse>',
        '  </SOAP-ENV:Body>',
        '</SOAP-ENV:Envelope>',
    ]).encode('utf-8')

    parser = Parser('https://example.com/api', lxmletree.fromstring(input))
    assert parser.get_header('authenticateResponse') is True
    assert len(parser._header) == 1
    assert parser.get_result('getZoneResponse') == {
        'id': 1,
        'user': 2,
        'name': 'example.com',
        'email': 'info@example.com',
        'ttl': 10800,
        'nameserver': 'ns1.hostserv.eu',
        'serial': '1234567890',
        'serialLastUpdate': 0,
        'refresh': 7200,
        'retry': 120,
        'expire': 1234567,
        'template': None,
        'ns3': 1,
        'records': [
            {
                'id': 3,
                'zone': 4,
                'type': 'A',
                'prefix': None,
                'target': '1.2.3.4',
                'ttl': 3600,
                'comment': None,
                'priority': None,
            },
            {
                'id': 5,
                'zone': 4,
                'type': 'A',
                'prefix': '*',
                'target': '1.2.3.5',
                'ttl': 3600,
                'comment': None,
                'priority': None,
            },
        ],
    }
    assert len(parser._body) == 1



# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.iosxr.tests.unit.compat.mock import patch
from ansible_collections.cisco.iosxr.tests.unit.modules.utils import (
    set_module_args,
)
from .iosxr_module import TestIosxrModule, load_fixture
from ansible_collections.cisco.iosxr.plugins.modules import iosxr_banner


class TestIosxrBannerModule(TestIosxrModule):

    module = iosxr_banner

    def setUp(self):
        super(TestIosxrBannerModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.cisco.iosxr.plugins.modules.iosxr_banner.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.cisco.iosxr.plugins.modules.iosxr_banner.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_is_cliconf = patch(
            "ansible_collections.cisco.iosxr.plugins.modules.iosxr_banner.is_cliconf"
        )
        self.is_cliconf = self.mock_is_cliconf.start()

    def tearDown(self):
        super(TestIosxrBannerModule, self).tearDown()

        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_is_cliconf.stop()

    def load_fixtures(self, commands=None):
        self.get_config.return_value = load_fixture("iosxr_banner_config.cfg")
        self.load_config.return_value = dict(diff=None, session="session")
        self.is_cliconf.return_value = True

    def test_iosxr_banner_login_create(self):
        set_module_args(dict(banner="login", text="test\nbanner\nstring"))
        commands = ["banner login test\nbanner\nstring"]
        self.execute_module(changed=True, commands=commands)

    def test_iosxr_banner_login_remove(self):
        set_module_args(dict(banner="login", state="absent"))
        commands = ["no banner login"]
        self.execute_module(changed=True, commands=commands)

    def test_iosxr_banner_fail_create(self):
        set_module_args(dict(banner="exec1", text="test\nbanner\nstring"))
        result = self.execute_module(failed=True, changed=True)
        self.assertEqual(
            result["msg"],
            "value of banner must be one of: login, motd, got: exec1",
        )

    def test_iosxr_banner_exec1_fail_remove(self):
        set_module_args(dict(banner="exec1", state="absent"))
        result = self.execute_module(failed=True, changed=True)
        self.assertIn(
            result["msg"],
            "value of banner must be one of: login, motd, got: exec1",
        )

    def test_iosxr_banner_motd_create(self):
        set_module_args(dict(banner="motd", text="test\nbanner\nstring"))
        commands = ["banner motd test\nbanner\nstring"]
        self.execute_module(changed=True, commands=commands)

    def test_iosxr_banner_motd_remove(self):
        set_module_args(dict(banner="motd", state="absent"))
        commands = ["no banner motd"]
        self.execute_module(changed=True, commands=commands)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for netaddr test plugin: resolvable
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.test.resolvable import (
    _resolvable,
)


class TestResolvable(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing argument
        with self.assertRaises(TypeError) as error:
            _resolvable()
        self.assertIn("argument", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        result = _resolvable(host="www.google.com")
        self.assertEqual(result, True)

        result = _resolvable(host="127.0.0.1")
        self.assertEqual(result, True)

        result = _resolvable(host="::1")
        self.assertEqual(result, True)

        result = _resolvable(host="foo.google.com")
        self.assertEqual(result, False)

        result = _resolvable(host="invalidhost")
        self.assertEqual(result, False)



# -*- coding: utf-8 -*-
# (c) 2015, Filipe Niero Felisbino <filipenf@gmail.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
  name: json_query
  short_description: Select a single element or a data subset from a complex data structure
  description:
    - This filter lets you query a complex JSON structure and iterate over it using a loop structure.
  positional: expr
  options:
    _input:
      description:
        - The JSON data to query.
      type: any
      required: true
    expr:
      description:
        - The query expression.
        - See U(http://jmespath.org/examples.html) for examples.
      type: string
      required: true
  requirements:
    - jmespath
'''

EXAMPLES = '''
- name: Define data to work on in the examples below
  ansible.builtin.set_fact:
    domain_definition:
      domain:
        cluster:
          - name: cluster1
          - name: cluster2
        server:
          - name: server11
            cluster: cluster1
            port: '8080'
          - name: server12
            cluster: cluster1
            port: '8090'
          - name: server21
            cluster: cluster2
            port: '9080'
          - name: server22
            cluster: cluster2
            port: '9090'
        library:
          - name: lib1
            target: cluster1
          - name: lib2
            target: cluster2

- name: Display all cluster names
  ansible.builtin.debug:
    var: item
  loop: "{{ domain_definition | community.general.json_query('domain.cluster[*].name') }}"

- name: Display all server names
  ansible.builtin.debug:
    var: item
  loop: "{{ domain_definition | community.general.json_query('domain.server[*].name') }}"

- name: Display all ports from cluster1
  ansible.builtin.debug:
    var: item
  loop: "{{ domain_definition | community.general.json_query(server_name_cluster1_query) }}"
  vars:
    server_name_cluster1_query: "domain.server[?cluster=='cluster1'].port"

- name: Display all ports from cluster1 as a string
  ansible.builtin.debug:
    msg: "{{ domain_definition | community.general.json_query('domain.server[?cluster==`cluster1`].port') | join(', ') }}"

- name: Display all ports from cluster1
  ansible.builtin.debug:
    var: item
  loop: "{{ domain_definition | community.general.json_query('domain.server[?cluster==''cluster1''].port') }}"

- name: Display all server ports and names from cluster1
  ansible.builtin.debug:
    var: item
  loop: "{{ domain_definition | community.general.json_query(server_name_cluster1_query) }}"
  vars:
    server_name_cluster1_query: "domain.server[?cluster=='cluster2'].{name: name, port: port}"

- name: Display all ports from cluster1
  ansible.builtin.debug:
    msg: "{{ domain_definition | to_json | from_json | community.general.json_query(server_name_query) }}"
  vars:
    server_name_query: "domain.server[?starts_with(name,'server1')].port"

- name: Display all ports from cluster1
  ansible.builtin.debug:
    msg: "{{ domain_definition | to_json | from_json | community.general.json_query(server_name_query) }}"
  vars:
    server_name_query: "domain.server[?contains(name,'server1')].port"
'''

RETURN = '''
  _value:
    description: The result of the query.
    type: any
'''

from ansible.errors import AnsibleError, AnsibleFilterError

try:
    import jmespath
    HAS_LIB = True
except ImportError:
    HAS_LIB = False


def json_query(data, expr):
    '''Query data using jmespath query language ( http://jmespath.org ). Example:
    - ansible.builtin.debug: msg="{{ instance | json_query(tagged_instances[*].block_device_mapping.*.volume_id') }}"
    '''
    if not HAS_LIB:
        raise AnsibleError('You need to install "jmespath" prior to running '
                           'json_query filter')

    # Hack to handle Ansible Unsafe text, AnsibleMapping and AnsibleSequence
    # See issue: https://github.com/ansible-collections/community.general/issues/320
    jmespath.functions.REVERSE_TYPES_MAP['string'] = jmespath.functions.REVERSE_TYPES_MAP['string'] + ('AnsibleUnicode', 'AnsibleUnsafeText', )
    jmespath.functions.REVERSE_TYPES_MAP['array'] = jmespath.functions.REVERSE_TYPES_MAP['array'] + ('AnsibleSequence', )
    jmespath.functions.REVERSE_TYPES_MAP['object'] = jmespath.functions.REVERSE_TYPES_MAP['object'] + ('AnsibleMapping', )
    try:
        return jmespath.search(expr, data)
    except jmespath.exceptions.JMESPathError as e:
        raise AnsibleFilterError('JMESPathError in json_query filter plugin:\n%s' % e)
    except Exception as e:
        # For older jmespath, we can get ValueError and TypeError without much info.
        raise AnsibleFilterError('Error in jmespath.search in json_query filter plugin:\n%s' % e)


class FilterModule(object):
    ''' Query filter '''

    def filters(self):
        return {
            'json_query': json_query
        }



# -*- coding: utf-8 -*-
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type


from ansible_collections.community.dns.plugins.module_utils.provider import (
    ProviderInformation,
)


class CustomProviderInformation(ProviderInformation):
    def __init__(self, txt_record_handling='decoded'):
        super(CustomProviderInformation, self).__init__()
        self._txt_record_handling = txt_record_handling

    def get_supported_record_types(self):
        return ['A']

    def get_zone_id_type(self):
        return 'str'

    def get_record_id_type(self):
        return 'str'

    def get_record_default_ttl(self):
        return 300

    def txt_record_handling(self):
        return self._txt_record_handling


class CustomProvideOptions(object):
    def __init__(self, option_dict):
        self._option_dict = option_dict

    def get_option(self, name):
        return self._option_dict.get(name)
